/**
  *
  * @author Kirk Pepperdine
  *
  */
package com.sams.ant.taskdef;

import org.apache.tools.ant.Task;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.types.Path;

import java.util.HashSet;
import java.util.ArrayList;
import java.util.Iterator;

public class ClassToSource extends Task {

    private Path sourcepath;
    private Path classpath;
    private boolean failonerror = true;

    public ClassToSource() {}

    public void setFailonerror(boolean value) {
        failonerror=value;
    }

    public Path createClassPath() {
        this.classpath = new Path(super.getProject());
        return classpath;
    }

    public Path createSourcePath() {
        this.sourcepath = new Path(super.getProject());
        return this.sourcepath;
    }

    public void execute() throws BuildException {
        log("Reconciling class and source files");
        try {
            execute0();
        } catch (BuildException be) {
            if (failonerror)
                throw be;
            else
                log(be.getMessage());
        }
        log("Reconciliation completed");
    }

    public void log(String[] messages) {
        for (int index = 0; index < messages.length; index++)
            log(messages[index]);
    }

    public void execute0() throws BuildException {
        HashSet classFiles = this.classFiles();
        Iterator iterator = this.sourceFiles().iterator();
        while (iterator.hasNext())
            classFiles.remove(iterator.next());

        if ( ! classFiles.isEmpty()) {
            Iterator i = classFiles.iterator();
            while (i.hasNext())
                log("No source found for " + (String)i.next());
            throw new BuildException(
                "classpath contains artifacts not found in sourcepath");
        } else
            log("No class files found");
    }

    private HashSet classFiles() {
        String[] filelist = this.classpath.list();
        HashSet set = new HashSet();

        for (int i = 0; i < filelist.length; i++)
            if ( filelist[i].endsWith(".class"))
                if (filelist[i].indexOf('$') < 0)
                    set.add(filelist[i].substring(
                            0, filelist[i].length() - 6));

        return set;

    }

    private ArrayList sourceFiles() {
        String[] filelist = this.sourcepath.list();
        ArrayList list = new ArrayList();
        String className;
        for (int i = 0; i < filelist.length; i++)
            if ( filelist[i].endsWith(".java"))
                list.add(filelist[i].substring(
                         0, filelist[i].length() - 5));
        return list;
    }
}
