/**
  *
  * @author Kirk Pepperdine
  *
  */
package com.sams.ant.taskdef;

import org.apache.tools.ant.Task;
import org.apache.tools.ant.BuildException;

import java.util.HashSet;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.StringTokenizer;


public class ClassToSource extends Task {

    private ArrayList sourcePath;
    private ArrayList classPath;
    private boolean failonerror = true;

    public ClassToSource() {}

    public void setSourcePath(String path) {
        sourcePath = parseSourcePath( path);
    }
    public void setClassPath(String path) {
        classPath = parseClassPath( path);
    }
    public void setFailonerror(boolean value) {
        failonerror=value;
    }

    public void execute() throws BuildException {
        log("Reconciling class and source files");
        try {
            execute0();
        } catch (BuildException be) {
            if (failonerror)
                throw be;
            else
                log(be.getMessage());
        }
        log("Reconciliation completed");
    }

    public void execute0() throws BuildException {
        HashSet classFiles = this.classFileSet();
        Iterator iterator = this.sourceFileSet().iterator();
        while (iterator.hasNext())
            classFiles.remove(iterator.next());

        if ( classFiles.size() != 0) {
            Iterator i = classFiles.iterator();
            while (i.hasNext())
                log("No source found for " +
                    (String)i.next());
            throw new BuildException(
                        "class with no source found"
                       );
        }
    }

    private ArrayList parse(String path) {
        ArrayList list = new ArrayList();
        return list;
    }

    private HashSet classFileSet() {
        HashSet set = new HashSet();
        Iterator i = classPath.iterator();
        ClassPathElement cpe;
        while (i.hasNext()) {
            cpe = (ClassPathElement)i.next();
            if ( cpe.exists())
                set.addAll(cpe.getClassFiles());
            else
                log(cpe.getElement() +
                    "does not exists, it will be ignored");
        }
        return set;
    }

    private HashSet sourceFileSet() {
        HashSet set = new HashSet();
        Iterator i = sourcePath.iterator();
        SourcePathElement spe;
        while (i.hasNext())
            set.addAll(((SourcePathElement)i.next()).
                       getSourceFiles());
        return set;
    }

    private ArrayList parse(String string,
                            String extension,
                            String delim) {
        ArrayList list = new ArrayList();
        StringTokenizer parser= new StringTokenizer(string,delim);
        while (parser.hasMoreElements())
            list.add(parser.nextElement());
        return list;
    }

    public ArrayList parseSourcePath(String path) {
        ArrayList list = new ArrayList();
        Iterator i = this.parse( path,
                                SourcePathElement.EXTENSION,
                                System.getProperty(
                                          "path.separator")).
                                       iterator();
        while (i.hasNext()) {
            SourcePathElement spe = new
                SourcePathElement((String)i.next());
            if ( spe.exists())
                list.add(spe);
            else
                log(spe.getElement() +
                    "does not exists, it will be ignored");
        }

        return list;
    }

    public ArrayList parseClassPath(String path) {
        ArrayList list = new ArrayList();
        Iterator i = this.parse( path,
                                 ClassPathElement.EXTENSION,
                                 System.getProperty(
                                          "path.separator")).
                                 iterator();
        while (i.hasNext()) {
            ClassPathElement cpe = new
                  ClassPathElement((String)i.next());
            if ( cpe.exists())
                list.add(cpe);
            else
                log(cpe.getElement() +
                    "does not exists, it will be ignored");
        }

        return list;
    }
}
