/**
  *
  * @author Kirk Pepperdine
  *
  */
package com.sams.ant.taskdef;

import java.io.File;
import java.util.HashSet;

public abstract class PathElement {

    File element;
    HashSet fileSet;
    String extension;

    public PathElement(String element, String extension) {
        this.element = new File(element);
        this.extension = extension;
    }

    public File getElement() { return element; }

    public boolean exists() {
        return element.exists();
    }

    public HashSet getFileSet() {
        if (fileSet != null) return fileSet;

        fileSet = new HashSet();
        String fileName;

        if ( element.isFile() && element.getName().
                                         endsWith(extension))
            fileSet.add(qualified(element));
        else {
            File[] paths = element.listFiles();
            for (int i = 0; i < paths.length; i++) {
                if ( paths[i].isDirectory())
                    getFilesIn(paths[i].getName(), paths[i]);
                else {
                    fileName = paths[i].getName();
                    if (fileName.endsWith(extension))
                        fileSet.add(qualified(paths[i]));
                }
            }
        }

        return fileSet;
    }

    private void getFilesIn( String packageName, File dir) {
        File[] contents = dir.listFiles();
        for (int i = 0; i < contents.length; i++) {
            if ( contents[i].isDirectory())
                getFilesIn( packageName +
                            "." + contents[i].getName(),
                            contents[i]);
            else if (contents[i].getName().
                                 endsWith(extension))
                    fileSet.add( packageName +
                                 "." +
                                 qualified(contents[i]));
        }
    }

    protected String qualified(File name) {
        String base = name.getName();
        return base.substring(0, base.indexOf( '.', 0));
    }
}
