package com.sams.ant.taskdef;

import com.sams.ant.typedef.DirectorySet;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.types.FileSet;

import java.util.HashSet;
import java.util.ArrayList;
import java.util.Iterator;

/**
 *
 * @author Kirk Pepperdine
 */

public class ClassToSource extends Task {

    private DirectorySet sourcepath;
    private DirectorySet classpath;
    private boolean failonerror = true;

    public ClassToSource() {}

    public void setFailonerror(boolean value) { failonerror=value; }

    public DirectorySet createClassPath() {
        this.classpath = new DirectorySet();
        classpath.setType(DirectorySet.CLASS);
        return classpath;
    }

    public DirectorySet createSourcePath() {
        this.sourcepath = new DirectorySet();
        sourcepath.setType(DirectorySet.SOURCE);
        return this.sourcepath;
    }

    public void execute() throws BuildException {
        log("Reconciling class and source files");
        try {
            execute0();
        } catch (BuildException be) {
            if (failonerror)
                throw be;
            else
                log(be.getMessage());
        }
        log("Reconciliation completed");
    }

    public void execute0() throws BuildException {
        HashSet classFiles = this.classFiles();
        Iterator iterator = this.sourceFiles().iterator();
        while (iterator.hasNext())
            classFiles.remove(iterator.next());

        if ( ! classFiles.isEmpty()) {
            Iterator i = classFiles.iterator();
            while (i.hasNext())
                log("No source found for " + (String)i.next());
            throw new
              BuildException(
                "classpath contains artifacts not found in sourcepath"
              );
        }
    }

    private HashSet classFiles() {
        String[] filelist = this.classpath.getFileList();
        HashSet set = new HashSet();

        for (int i = 0; i < filelist.length; i++)
            if (filelist[i].indexOf('$') < 0)
                set.add(filelist[i].substring(
                        0, filelist[i].length() - 6));

        return set;

    }

    private ArrayList sourceFiles() {
        String[] filelist = this.sourcepath.getFileList();
        ArrayList list = new ArrayList();
        String className;
        for (int i = 0; i < filelist.length; i++)
            list.add(filelist[i].substring(
                     0, filelist[i].length() - 5));

        return list;
    }
}
