/*
 * Channel
 *
 * @author kpepperdine
 * Date: May 30, 2002
 * Time: 10:47:14 AM
 */

package com.sams.ant.channel;

import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicSession;
import javax.naming.Context;
import javax.naming.InitialContext;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

public abstract class Channel {

    private static final String CHANNEL_PROPERTIES_FILE_NAME = "channel.properties";
    private static final String INITIAL_CONTEXT_FACTORY_PATH = "com.sams.ant.channel.Channel.initial.context";
    private static final String PROTOCAL_PATH = "com.sams.ant.channel.Channel.protocal";
    private static final String URL_PATH = "com.sams.ant.channel.Channel.url";

    private TopicSession session;
    private InitialContext jndi;
    private TopicConnection connection;
    private Topic topic;

    protected TopicSession getSession() {
        return this.session;
    }

    protected Topic getTopic() {
        return this.topic;
    }

    protected TopicConnection getConnection() {
        return this.connection;
    }

    private Properties getChannelProperties() throws IOException {
        Properties channelProperties = new Properties();
        InputStream stream = this.getClass().getResourceAsStream(CHANNEL_PROPERTIES_FILE_NAME);
        channelProperties.load(stream);
        stream.close();
        return channelProperties;
    }

    public void open(String host, String port, String userid, String password, String topicName) throws ChannelException {

        try {
            Properties env = new Properties();
            Properties configurations = this.getChannelProperties();
            String initialContextFactory = (String)configurations.get(INITIAL_CONTEXT_FACTORY_PATH);
            String protocal = (String)configurations.get(PROTOCAL_PATH);
            String url = (String)configurations.get(URL_PATH);
            env.put(Context.INITIAL_CONTEXT_FACTORY,initialContextFactory);
            env.put(Context.PROVIDER_URL,protocal + host + ":" + port + url);
            this.jndi = new InitialContext(env);
            this.connection = ((TopicConnectionFactory)jndi.lookup("TopicConnectionFactory")).createTopicConnection(userid,password);
            this.session = connection.createTopicSession(false,Session.AUTO_ACKNOWLEDGE);
            this.topic = (Topic)jndi.lookup(topicName);
        } catch (Exception e) {
            throw new ChannelException(e);
        }
    }

    public void close() {
        try {
            this.jndi.close();
            this.connection.close();
            this.session.close();
        } catch (Exception e) {
            // eat this as it doesn't matter at this point.
        } finally {
            this.session = null;
            this.connection = null;
            this.jndi = null;
        }
    }
}
