package com.sams.ant.taskdef;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.Mkdir;
import org.apache.tools.ant.taskdefs.Cvs;
import org.apache.tools.ant.taskdefs.Ant;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

/**
  *
  * @author Kirk Pepperdine
  *
  */

public class Release extends Task {

    private static String fileSeparator =
                   System.getProperty("file.separator");

    private boolean failonerror = true;
    private String file;
    private File sourceDir;

    public Release() {}

    public void setFailonerror(boolean value) {
        failonerror=value;
    }

    public void setFile(String value) {
        this.file = value;
    }

    public void setSourceDir(String value) {
        sourceDir = new File(value);
    }

    public void execute() throws BuildException {
        log("Release: " + this.file);
        try {
            execute0();
        } catch (BuildException be) {
            if (failonerror)
                throw be;
            else
                log(be.getMessage());
        }
    }

    public void execute0() throws BuildException {
        try {
            this.createSourceDir();
            BufferedReader configuration = new BufferedReader(new FileReader(this.file));
            String configComponent;
            while ( ( configComponent = configuration.readLine()) != null)
                processConfigComponent(configComponent);
        } catch (Exception e) {
            throw new BuildException(e);
        }
    }

    private void processConfigComponent(String component) throws BuildException {
        if ( ( component.length() == 0) ||
             ( component.charAt(0) == '#') ||
             (component.indexOf(",") < 0))
            return;

        StringTokenizer st = new StringTokenizer(component,",");
        if ( ! st.hasMoreTokens())
            return;

        String moduleName = st.nextToken();
        String version = st.nextToken();
        String buildFile = st.nextToken();
        this.getSource(moduleName, version);

        while (st.hasMoreTokens()) {
            this.perform(moduleName, buildFile, st.nextToken());
        }

    }

    private void executeTask(Task task) throws BuildException {
        task.setProject(this.getProject());
        task.setLocation(this.getLocation());
        task.setOwningTarget(this.getOwningTarget());
        task.init();
        task.execute();
    }

    private void getSource(String moduleName, String version) throws BuildException {
        Cvs task = new Cvs();
        task.setCommand("checkout");
        task.setPackage(moduleName);
        task.setTag(version);
        task.setDest(this.sourceDir);
        this.executeTask(task);
    }

    // make the output directory
    private void createSourceDir() throws BuildException {
        if ( ( sourceDir.exists()) && ( ! sourceDir.isDirectory()))
            throw new BuildException(sourceDir.toString() + "is not a directory");

        Mkdir task = new Mkdir();
        task.setDir(this.sourceDir);
        this.executeTask(task);
    }

    private void perform(String componentName,
                         String buildFile,
                         String taskName) throws BuildException {
        Ant task = new Ant();
        task.setInheritAll(false);
        task.setTarget(taskName);
        try {
            task.setAntfile(this.sourceDir.getCanonicalPath() +
                            this.fileSeparator + componentName +
                            this.fileSeparator + buildFile );
        } catch (IOException e) {
            throw new BuildException(e);
        }
        this.executeTask(task);
    }
}
