/*
 * Flight.java
 *
 * Created on 5. februar 2001, 20:40
 */

package com.bekk.dbcdemo;

/**
 * This interface describes a flight
 *
 * @author  <a href="mailto:aslak.hellesoy@bekk.no">Aslak Hellesy</a>
 * @invariant getPassengers() >= 0 // never a negative number of passengers
 */
public interface Flight {

	public static final int MAX_HAND_LUGGAGE_WEIGHT = 5;
	public static final int MAX_PASSENGERS = 100;

	/**
	 * Accepts a passenger and brings the passenger to its final destination
	 *
	 * @param passenger the passenger to bring
	 * @return the name of the destination
	 *
	 * @pre passenger != null
	 * @pre passenger.getDestination() != null
	 * @pre passenger.getHandLuggageWeight() <= MAX_HAND_LUGGAGE_WEIGHT
	 * @pre passenger.getHandLuggageWeight() >= 0
	 * @pre passenger.hasTicket()
	 * @pre getPassengers() < MAX_PASSENGERS
	 *
	 * @post return != null
	 * @post getPassengers() == getPassengers()@pre + 1
	 */
	public abstract String accept( Passenger passenger );

	/**
	 * Returns the name of the flight
	 *
	 * @return the name of the airline
	 * @post return != null
	 */
	public abstract String getName();

	/**
	 * Returns the number of passengers checked in for this flight
	 *
	 * @return the number of passengers
	 */
	public abstract int getPassengers();
}
